
#toolbar{
 padding-bottom: 10px;
 }

#header-right li{
 list-style-type: none;
 }

.navbar a.brand {
    display: block;
    padding: 15px 20px;
}

div#header-center {
    padding: 0 0 0;
}

.navbar .nav > li > a {
   background: none;
}