"""cubicweb-brainomics application package

Cube for the Brainomics Project, see http://www.brainomics.net/
"""

ALL_MEASURES = ('GenericMeasure', 'QuestionnaireRun', 'GenericTestRun', 'Scan', 'GenomicMeasure')
COMMENTED_ENTITIES = ('Study', 'Subject', 'Center', 'Device', 'Scan', 'GenomicMeasure',
                      'QuestionnaireRun', 'Questionnaire', 'Question')
WIKI_ENTITIES = ('Question', 'Questionnaire', 'Study', 'ScoreDefinition')
